/*
 *  Copyright (C) 2005 Steve Harris
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: datatypes.c,v 1.5 2005/09/30 08:38:47 theno23 Exp $
 */

#include <stdlib.h>

#include "tstore.h"

char *ts_feature_names[TS_FEATURE_LAST+1] = {
        "Standards compliance",
        "Inference",
	"Provenance",
	"Retreive"
};

void ts_query_free(ts_query *q)
{
    GSList *it;

    if (q) {
	free(q->sql_query);
	free(q->error_str);
	free(q->col_desc);
	free(q->bindings);
	if (q->rquery) rasqal_free_query(q->rquery);
	raptor_free_uri(q->base);

	for (it = q->sql_setup; it; it = it->next) {
	    free(it->data);
	}
	g_slist_free(q->sql_setup);

	for (it = q->sql_pre_query; it; it = it->next) {
	    free(it->data);
	}
	g_slist_free(q->sql_pre_query);

	for (it = q->sql_cleanup; it; it = it->next) {
	    free(it->data);
	}
	g_slist_free(q->sql_cleanup);

	free(q);
    }
}

char *ts_rel_query_to_string(ts_rel_query *q, char *prefix, char *postfix)
{
    if (!q) {
	return "(null)";
    }

    return g_strdup_printf("%s%s%s%s%s%s",
		prefix,
		q->map ? q->map : "",
		q->project ? q->project : "",
		q->join ? q->join : "",
		q->select ? q->select : "",
		postfix);
}

void ts_rel_query_free(ts_rel_query *q)
{
    if (q) {
	free(q->map);
	free(q->project);
	free(q->join);
	free(q->select);
	free(q);
    }
}

/* vi:set ts=8 sts=4 sw=4: */
