#ifndef DB_CONFIG_H
#define DB_CONFIG_H

#include <mysql.h>
#include <rasqal.h>

#include "datatypes.h"

#define TS_CF_VAL_LENGTH 224 /* the maximum length of config values */
#define TS_CF_VAL_SIZE   256 /* a good size to allocate for returned values */

/* return the version number of the table layout for the given database */
int ts_config_get_table_version(MYSQL *db);

/* set the version number of the table layout for the given database and
 * version */
void ts_config_set_table_version(MYSQL *db, int version);

/* get the value of a setting. If value is non NULL then it will have the value
 * of that setting written over it, and value will be returned. if its NULL
 * then space will be allocated, the value written to that, and a pointer to
 * the string returned. If there is no value for the given key, then NULL will
 * be returned. */
char *ts_config_get_value(ts_connection *c, const char *key, const char *value);

/* update or replace the key with the given value */
int ts_config_set_value(ts_connection *c, const char *key, const char *value);

/* vi:set ts=8 sts=4 sw=4: */

#endif
