/*
 *  Copyright (C) 2005 Steve Harris
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: ts-explain.c,v 1.10 2005/12/06 13:57:58 theno23 Exp $
 */

#include <stdio.h>
#include <unistd.h>
#include <libgen.h>
#include <string.h>
#include <rasqal.h>

#include "tstore.h"
#include "cmdline-parser.h"

int main(int argc, char*argv[])
{
    ts_connection *con;
    int help = 0;
    char *optstring = TSP_OPTSTR "l:O:f:";
    int c, opt_index = 0;
    static struct option long_options[] = {
        TSP_LONGOPTS,
        { "language",     1, 0, 'l' },
        { "optimise",     1, 0, 'O' },
        { "format",       1, 0, 'f' },
	{ 0, 0, 0, 0 }
    };
    GSList *it;
    char *query_lang = "sparql";
    char *query_string;
    int optimisation = -1;
    int comp;

    ts_init();

    while ((c = getopt_long (argc, argv, optstring, long_options, &opt_index)) != -1) {
        if (!tsp_handle_option(c, optarg)) {
            switch (c) {
            case 'l':
                query_lang = optarg;
                break;
            case 'O':
                optimisation = atoi(optarg);
                break;
	    case 'f':
		/* ignored - just in here for compat. with ts-query */
		break;
            case 'h':
                help++;
                break;
            default:
                help++;
                break;
            }
        }
    }

    if (help || argc - optind > 1 || argc == 1) {
	int i;
	const char *lang;
	const char *label;

	rasqal_init();

        fprintf(stderr, "Usage: %s "TSP_OPTIONS" <query>\n" TSP_HELP, basename(argv[0]));
	fprintf(stderr, "   -l, --language  use language for this explanation\n");
	fprintf(stderr, "    where language is one of:");
	for (i=0; 1; i++) {
	    if (rasqal_languages_enumerate(i, &lang, &label, NULL)) {
		break;
	    }
	    fprintf(stderr, " %s", lang);
	}
	fprintf(stderr, "\n");
	fprintf(stderr, "See man pages for more details\n");
        return 1;
    }

    query_string = argv[optind];

    con = tsp_connect(NULL);
    if (optimisation >= 0) {
	ts_connection_set_optimisation(con, optimisation);
    }
    
    ts_init();

    if (!strcmp(query_string, "-")) {
	char qs[10240];
	char *pos = qs;
	int ret;

	do {
	    ret = read(0, pos, sizeof(qs)-1-strlen(qs));
	    pos += strlen(pos);
	} while (ret > 0);
	*pos = '\0';
	query_string = qs;
    }

    ts_query *tq = ts_query_prepare(con, query_lang, query_string,
				  "3store:default#");
    if (tq->error_str) {
	printf("Error: %s\n", tq->error_str);
    } else {
	comp = ts_query_complexity(con, tq);
	printf("Complexity: %d\n", comp);
	for (it = tq->sql_setup; it; it = it->next) {
	    printf("%s;\n", (char *)it->data);
	}
	for (it = tq->sql_pre_query; it; it = it->next) {
	    printf("%s;\n", (char *)it->data);
	}
	printf("%s;\n", tq->sql_query);
	for (it = tq->sql_cleanup; it; it = it->next) {
	    printf("%s;\n", (char *)it->data);
	}
    }

    ts_query_free(tq);
    ts_disconnect(con);
    ts_finish();

    return 0;
}

/* vi:set ts=8 sts=4 sw=4: */
