/*
 *  Copyright (C) 2005 Steve Harris
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: ts-remove.c,v 1.7 2005/11/10 17:58:46 theno23 Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <libgen.h>
#include <rasqal.h>

#include "tstore.h"
#include "results.h"
#include "cmdline-parser.h"

int main(int argc, char*argv[])
{
    ts_connection *con;
    int help = 0;
    char *optstring = "-" TSP_OPTSTR "afv";
    int c, opt_index = 0;
    int verbosity = 0;
    static struct option long_options[] = {
        TSP_LONGOPTS,
        { "all", 0, 0, 'a' },
        { "force", 0, 0, 'f' },
        { "verbose", 0, 0, 'v' },
        { 0, 0, 0, 0 }
    };
    int all = 0;
    int force = 0;
    int done = 0;
    char *uri;

    ts_init();

    while ((c = getopt_long (argc, argv, optstring, long_options, &opt_index)) != -1) {
        if (!tsp_handle_option(c, optarg)) {
            switch (c) {
	    case 'f':
		force = 1;
		break;
	    case 'a':
		all = 1;
		break;
	    case 'v':
		verbosity++;
		break;
	    case 1:
		con = tsp_connect(NULL);
		uri = ts_path_to_uri(optarg);
		ts_erase(con, uri, TS_MODEL_DATA, verbosity, stdout);
		free(uri);
		ts_disconnect(con);
		con = NULL;
		done++;
		break;
            default:
                help++;
                break;
            }
        }
    }
    con = tsp_connect(NULL);

    if (help || (!done && !all)) {
        fprintf(stderr, "Usage: %s "TSP_OPTIONS" [-a] [-v] "
	        "[-f] [<model> ...]\n" TSP_HELP, basename(argv[0]));
	fprintf(stderr, "See man pages for more details\n");
        return 1;
    }

    if (all && force) {
	printf("Erasing all models\n");
	if (ts_erase_all(con, verbosity, stdout)) {
	    printf("Failed\n");
	}
    } else if (all) {
	fprintf(stderr, "Use --force to erase all data in the database\n");
    }

    ts_disconnect(con);
    ts_finish();

    return 0;
}

/* vi:set ts=8 sts=4 sw=4: */
