#ifndef TSTORE_H
#define TSTORE_H

#include <rasqal.h>

#include "symtab.h"
#include "datatypes.h"
#include "hash.h"
#include "rdf_constants.h"
#include "util.h"
#include "db-config.h"
#include "import.h"
#include "assert.h"

#define DEFAULT_OPT_LVL        2
#define DEFAULT_LOCK_TABLES    1

#define TS_MODEL_HISTORY       1
#define TS_MODEL_DATA          2
#define TS_MODEL_DELAY_CLEANUP 4

#define TS_ORDER_EXPRESSION_ASC  (RASQAL_LITERAL_LAST+23)
#define TS_ORDER_EXPRESSION_DESC (RASQAL_LITERAL_LAST+24)

/* Setup functions */

void ts_init();

void ts_finish();

ts_connection *ts_connect(const char *host, const char *db,
                          const char *user, const char *passwd);

void ts_connection_set_optimisation(ts_connection *c, int opt);

void ts_disconnect(ts_connection *c);

/* Query execution functions */

ts_query *ts_query_prepare(ts_connection *c, const char *lang,
	const char *query, const char *base_uri);

int ts_query_complexity(ts_connection *c, ts_query *q);

ts_result *ts_query_execute(ts_connection *c, ts_query *q);

ts_row *ts_result_fetch_row(ts_result *res);

ts_binding *ts_row_get_binding(ts_row *row, int column);

ts_binding *ts_row_get_binding_by_name(ts_row *row, const char *name);

/* Low level functions */

int ts_assert_triple(ts_connection *c, ts_hash model, const char *subj,
		     const char *pred, const char *obj,
		     const ts_obj_type type, int dt, int lang);

ts_query *ts_sql_compile(ts_query *q, ts_connection *c, rasqal_query *rq);

ts_hash ts_ensure_resource(ts_connection *c, const char *uri);
ts_hash ts_ensure_literal(ts_connection *c, char *lex, int dt, int lang);

/* takes a date format string, and returns a normalised datestring in UTC if
 * possible. The timedt argument should point to a 512 char buffer that may get
 * scribbled on to store the result  */
char *ts_normalise_date(ts_connection *c, int dt, char *buffer, char *timedt);

/* error codes */
#define TS_SUCCESS   0
#define TS_BAD_TYPE  1
#define TS_BAD_SQL   2
#define TS_ALLREADY  3

#endif

/* vi:set ts=8 sts=4 sw=4: */
