/*
 *  Copyright (C) 2005 Steve Harris
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: util.c,v 1.11 2005/11/16 18:46:51 theno23 Exp $
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <mysql.h>

#include "util.h"
#include "timing.h"

#define TMP_SIZE 256

void ts_list_replace_str(GSList *l, const char *from, const char *to)
{
    GSList *it;
    int len = strlen(from);

    if (len != strlen(to)) {
	fprintf(stderr, "lib3store error: length '%s' != length '%s'\n",
		from, to);
    
	return;
    }

    for (it = l; it; it=it->next) {
	char *p = strstr((char *)it->data, from);

	if (p) {
	    memcpy(p, to, len);
	}
    }
}

int ts_sql_count(void *db, const char *q)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    int ret;
    int count_q = 0;

    ts_mysql_query(db, q);
    if (strstr(q, "COUNT")) {
	count_q = 1;
    }
    res = mysql_store_result(db);
    if (!res) {
	return 0;
    }
    if (!count_q) {
	ret = mysql_num_rows(res);
	mysql_free_result(res);

	return ret;
    }
    row = mysql_fetch_row(res);

    if (!row) {
	fprintf(stderr, "%s - doesnt look like a valid COUNT query\n", q);

	return -1;
    }

    ret = atoi(row[0]);
    mysql_free_result(res);

    return ret;
}

int ts_mysql_query_intl(MYSQL *db, const char *query, const char *file, const int line)
{
    int ret = mysql_query(db, query);

    if (ret) {
	fprintf(stderr, "SQL error %d: %s\nat %s:%d\n", 
		mysql_errno(db), mysql_error(db), file, line);
    }

    return ret;
}

char *ts_path_to_uri(char *path)
{
    char *ret;

    if (!path || !(*path)) return NULL;

    if (!strchr(path, ':')) {
	if (*path == '/') {
	    ret = g_strdup_printf("file://%s", path);
	} else {
	    ret = g_strdup_printf("file:%s", path);
	}
    } else {
	ret = strdup(path);
    }

    return ret;
}

uint32_t ts_claim_ai_vals(ts_connection *c, char *table, char *column, int size)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    char tmp[TMP_SIZE];
    int max;
    int new_ai;

    TIME(NULL);
    snprintf(tmp, TMP_SIZE-1, "LOCK TABLE %s WRITE", table);
    ts_mysql_query(c->db, tmp);
    snprintf(tmp, TMP_SIZE-1, "SELECT max(%s) FROM %s", column, table);
    ts_mysql_query(c->db, tmp);
    res = mysql_use_result(c->db);
    row = mysql_fetch_row(res);
    max = atoi(row[0]);
    mysql_free_result(res);
    new_ai = max + size + 1;
    snprintf(tmp, TMP_SIZE-1, "ALTER TABLE %s AUTO_INCREMENT=%d", table, new_ai);
    ts_mysql_query(c->db, tmp);
    ts_mysql_query(c->db, "UNLOCK TABLES");
    TIME("claim_ai_vals");

    return max + 1;
}

/* this is platoform specific, needs porting to other platforms */
int ts_physical_ram(long long *total_mem, long long *free_mem)
{
    FILE *mi = fopen("/proc/meminfo", "r");
    char buf[TMP_SIZE];
    int retval = 0;

    /* some vague guesses incase we cant find the real values */
    *total_mem = 500000000;
    *free_mem = (*total_mem) / 2;
	
    if (!mi) {
	fprintf(stderr, "warning: cannot read memory stats, guessing buffer sizes\n");

	return -1;
    }

    while (fgets(buf, sizeof(buf), mi) != NULL) {
	if (!strncmp(buf, "MemTotal:", 9)) {
	    if (sscanf(buf, "%*s %Lu", total_mem) != 1) {
		fprintf(stderr, "warning: failed to parse MemTotal line\n");
		retval--;
	    } else {
		*total_mem *= 1024;
	    }
	}
	if (!strncmp(buf, "MemFree:", 8)) {
	    if (sscanf(buf, "%*s %Lu", free_mem) != 1) {
		fprintf(stderr, "warning: failed to parse MemFree line\n");
		retval--;
	    } else {
		*free_mem *= 1024;
	    }
	}
    }

    return retval;
}

void ts_sql_processlist(ts_connection *c)
{
    MYSQL_RES *res;
    MYSQL_ROW row;

    ts_mysql_query(c->db, "SHOW PROCESSLIST");
    res = mysql_use_result(c->db);
    while ((row = mysql_fetch_row(res))) {
	printf("mode %s\nactn %s\n", row[6], row[7]);
    }
    mysql_free_result(res);
}

void ts_entry_free(gpointer data)
{
    free(data);
}

/* vi:set ts=8 sts=4 sw=4: */
